@@ -4,7 +4,7 @@
  * Copyright (C) 1996-2001 Andrew Tridgell <tridge@samba.org>
  * Copyright (C) 1996 Paul Mackerras
  * Copyright (C) 2001, 2002 Martin Pool <mbp@samba.org>
- * Copyright (C) 2003-2009 Wayne Davison
+ * Copyright (C) 2003-2013 Wayne Davison
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,27 +21,30 @@
  */
 
 #include "rsync.h"
-#include "ifuncs.h"
+#include "inums.h"
 #include "io.h"
 #if defined CONFIG_LOCALE && defined HAVE_LOCALE_H
 #include <locale.h>
 #endif
 
-extern int verbose;
 extern int dry_run;
 extern int list_only;
+extern int io_timeout;
 extern int am_root;
 extern int am_server;
 extern int am_sender;
 extern int am_daemon;
 extern int inc_recurse;
 extern int blocking_io;
+extern int always_checksum;
 extern int remove_source_files;
+extern int output_needs_newline;
 extern int need_messages_from_generator;
 extern int kluge_around_eof;
-extern int do_stats;
 extern int got_xfer_error;
+extern int msgs2stderr;
 extern int module_id;
+extern int read_only;
 extern int copy_links;
 extern int copy_dirlinks;
 extern int copy_unsafe_links;
@@ -62,31 +65,44 @@ extern int whole_file;
 extern int read_batch;
 extern int write_batch;
 extern int batch_fd;
+extern int sock_f_in;
+extern int sock_f_out;
 extern int filesfrom_fd;
 extern int connect_timeout;
+extern int send_msgs_to_gen;
 extern dev_t filesystem_dev;
 extern pid_t cleanup_child_pid;
+extern size_t bwlimit_writemax;
 extern unsigned int module_dirlen;
+extern BOOL flist_receiving_enabled;
+extern BOOL shutting_down;
+extern int basis_dir_cnt;
 extern struct stats stats;
+extern char *stdout_format;
+extern char *logfile_format;
 extern char *filesfrom_host;
 extern char *partial_dir;
 extern char *dest_option;
-extern char *basis_dir[MAX_BASIS_DIRS+1];
 extern char *rsync_path;
 extern char *shell_cmd;
 extern char *batch_name;
 extern char *password_file;
+extern char *backup_dir;
 extern char curr_dir[MAXPATHLEN];
+extern char backup_dir_buf[MAXPATHLEN];
+extern char *basis_dir[MAX_BASIS_DIRS+1];
 extern struct file_list *first_flist;
-extern struct filter_list_struct daemon_filter_list;
+extern filter_rule_list daemon_filter_list;
 
 uid_t our_uid;
+gid_t our_gid;
 int am_receiver = 0;  /* Only set to 1 after the receiver/generator fork. */
 int am_generator = 0; /* Only set to 1 after the receiver/generator fork. */
 int local_server = 0;
 int daemon_over_rsh = 0;
 mode_t orig_umask = 0;
 int batch_gen_fd = -1;
+int sender_keeps_checksum = 0;
 
 /* There's probably never more than at most 2 outstanding child processes,
  * but set it higher, just in case. */
@@ -116,7 +132,7 @@ static void show_malloc_stats(void);
 pid_t wait_process(pid_t pid, int *status_ptr, int flags)
 {
 	pid_t waited_pid;
-	
+
 	do {
 		waited_pid = waitpid(pid, status_ptr, flags);
 	} while (waited_pid == -1 && errno == EINTR);
@@ -169,6 +185,30 @@ static void wait_process_with_flush(pid_t pid, int *exit_code_ptr)
 		*exit_code_ptr = WEXITSTATUS(status);
 }
 
+void write_del_stats(int f)
+{
+	if (read_batch)
+		write_int(f, NDX_DEL_STATS);
+	else
+		write_ndx(f, NDX_DEL_STATS);
+	write_varint(f, stats.deleted_files - stats.deleted_dirs
+		      - stats.deleted_symlinks - stats.deleted_devices
+		      - stats.deleted_specials);
+	write_varint(f, stats.deleted_dirs);
+	write_varint(f, stats.deleted_symlinks);
+	write_varint(f, stats.deleted_devices);
+	write_varint(f, stats.deleted_specials);
+}
+
+void read_del_stats(int f)
+{
+	stats.deleted_files = read_varint(f);
+	stats.deleted_files += stats.deleted_dirs = read_varint(f);
+	stats.deleted_files += stats.deleted_symlinks = read_varint(f);
+	stats.deleted_files += stats.deleted_devices = read_varint(f);
+	stats.deleted_files += stats.deleted_specials = read_varint(f);
+}
+
 /* This function gets called from all 3 processes.  We want the client side
  * to actually output the text, but the sender is the only process that has
  * all the stats we need.  So, if we're a client sender, we do the report.
@@ -185,7 +225,7 @@ static void handle_stats(int f)
 	total_read = stats.total_read;
 	total_written = stats.total_written;
 
-	if (do_stats && verbose > 1) {
+	if (INFO_GTE(STATS, 3)) {
 		/* These come out from every process */
 		show_malloc_stats();
 		show_flist_stats();
@@ -239,13 +279,39 @@ static void handle_stats(int f)
 	}
 }
 
+static void output_itemized_counts(const char *prefix, int *counts)
+{
+	static char *labels[] = { "reg", "dir", "link", "dev", "special" };
+	char buf[1024], *pre = " (";
+	int j, len = 0;
+	int total = counts[0];
+	if (total) {
+		counts[0] -= counts[1] + counts[2] + counts[3] + counts[4];
+		for (j = 0; j < 5; j++) {
+			if (counts[j]) {
+				len += snprintf(buf+len, sizeof buf - len - 2,
+					"%s%s: %s",
+					pre, labels[j], comma_num(counts[j]));
+				pre = ", ";
+			}
+		}
+		buf[len++] = ')';
+	}
+	buf[len] = '\0';
+	rprintf(FINFO, "%s: %s%s\n", prefix, comma_num(total), buf);
+}
+
 static void output_summary(void)
 {
-	if (do_stats) {
+	if (INFO_GTE(STATS, 2)) {
 		rprintf(FCLIENT, "\n");
-		rprintf(FINFO,"Number of files: %d\n", stats.num_files);
-		rprintf(FINFO,"Number of files transferred: %d\n",
-			stats.num_transferred_files);
+		output_itemized_counts("Number of files", &stats.num_files);
+		if (protocol_version >= 29)
+			output_itemized_counts("Number of created files", &stats.created_files);
+		if (protocol_version >= 31)
+			output_itemized_counts("Number of deleted files", &stats.deleted_files);
+		rprintf(FINFO,"Number of regular files transferred: %s\n",
+			comma_num(stats.xferred_files));
 		rprintf(FINFO,"Total file size: %s bytes\n",
 			human_num(stats.total_size));
 		rprintf(FINFO,"Total transferred file size: %s bytes\n",
@@ -258,11 +324,11 @@ static void output_summary(void)
 			human_num(stats.flist_size));
 		if (stats.flist_buildtime) {
 			rprintf(FINFO,
-				"File list generation time: %.3f seconds\n",
-				(double)stats.flist_buildtime / 1000);
+				"File list generation time: %s seconds\n",
+				comma_dnum((double)stats.flist_buildtime / 1000, 3));
 			rprintf(FINFO,
-				"File list transfer time: %.3f seconds\n",
-				(double)stats.flist_xfertime / 1000);
+				"File list transfer time: %s seconds\n",
+				comma_dnum((double)stats.flist_xfertime / 1000, 3));
 		}
 		rprintf(FINFO,"Total bytes sent: %s\n",
 			human_num(total_written));
@@ -270,15 +336,15 @@ static void output_summary(void)
 			human_num(total_read));
 	}
 
-	if (verbose || do_stats) {
+	if (INFO_GTE(STATS, 1)) {
 		rprintf(FCLIENT, "\n");
 		rprintf(FINFO,
 			"sent %s bytes  received %s bytes  %s bytes/sec\n",
 			human_num(total_written), human_num(total_read),
 			human_dnum((total_written + total_read)/(0.5 + (endtime - starttime)), 2));
-		rprintf(FINFO, "total size is %s  speedup is %.2f%s\n",
+		rprintf(FINFO, "total size is %s  speedup is %s%s\n",
 			human_num(stats.total_size),
-			(double)stats.total_size / (total_written+total_read),
+			comma_dnum((double)stats.total_size / (total_written+total_read), 2),
 			write_batch < 0 ? " (BATCH ONLY)" : dry_run ? " (DRY RUN)" : "");
 	}
 
@@ -299,7 +365,7 @@ static void show_malloc_stats(void)
 
 	rprintf(FCLIENT, "\n");
 	rprintf(FINFO, RSYNC_NAME "[%d] (%s%s%s) heap statistics:\n",
-		getpid(), am_server ? "server " : "",
+		(int)getpid(), am_server ? "server " : "",
 		am_daemon ? "daemon " : "", who_am_i());
 	rprintf(FINFO, "  arena:     %10ld   (bytes from sbrk)\n",
 		(long)mi.arena);
@@ -332,7 +398,7 @@ static pid_t do_cmd(char *cmd, char *machine, char *user, char **remote_argv, in
 		    int *f_in_p, int *f_out_p)
 {
 	int i, argc = 0;
-	char *args[MAX_ARGS];
+	char *args[MAX_ARGS], *need_to_free = NULL;
 	pid_t pid;
 	int dash_l_set = 0;
 
@@ -343,7 +409,7 @@ static pid_t do_cmd(char *cmd, char *machine, char *user, char **remote_argv, in
 			cmd = rsh_env;
 		if (!cmd)
 			cmd = RSYNC_RSH;
-		cmd = strdup(cmd); /* MEMORY LEAK */
+		cmd = need_to_free = strdup(cmd);
 		if (!cmd)
 			goto oom;
 
@@ -440,7 +506,7 @@ static pid_t do_cmd(char *cmd, char *machine, char *user, char **remote_argv, in
 
 	args[argc] = NULL;
 
-	if (verbose > 3) {
+	if (DEBUG_GTE(CMD, 2)) {
 		for (i = 0; i < argc; i++)
 			rprintf(FCLIENT, "cmd[%d]=%s ", i, args[i]);
 		rprintf(FCLIENT, "\n");
@@ -479,6 +545,9 @@ static pid_t do_cmd(char *cmd, char *machine, char *user, char **remote_argv, in
 			send_protected_args(*f_out_p, args);
 	}
 
+	if (need_to_free)
+		free(need_to_free);
+
 	return pid;
 
   oom:
@@ -505,7 +574,7 @@ static char *get_local_name(struct file_list *flist, char *dest_path)
 	int statret;
 	char *cp;
 
-	if (verbose > 2) {
+	if (DEBUG_GTE(RECV, 1)) {
 		rprintf(FINFO, "get_local_name count=%d %s\n",
 			file_total, NS(dest_path));
 	}
@@ -526,7 +595,7 @@ static char *get_local_name(struct file_list *flist, char *dest_path)
 		if ((*dest_path != '.' || dest_path[1] != '\0')
 		 && (check_filter(&daemon_filter_list, FLOG, dest_path, 0) < 0
 		  || check_filter(&daemon_filter_list, FLOG, dest_path, 1) < 0)) {
-			rprintf(FERROR, "skipping daemon-excluded destination \"%s\"\n",
+			rprintf(FERROR, "ERROR: daemon has excluded destination \"%s\"\n",
 				dest_path);
 			exit_cleanup(RERR_FILESELECT);
 		}
@@ -581,7 +650,7 @@ static char *get_local_name(struct file_list *flist, char *dest_path)
 			exit_cleanup(RERR_SYNTAX);
 		}
 
-		if (mkdir_defmode(dest_path) != 0) {
+		if (do_mkdir(dest_path, ACCESSPERMS) != 0) {
 			rsyserr(FERROR, errno, "mkdir %s failed",
 				full_fname(dest_path));
 			exit_cleanup(RERR_FILEIO);
@@ -591,7 +660,7 @@ static char *get_local_name(struct file_list *flist, char *dest_path)
 		 && strcmp(flist->files[flist->low]->basename, ".") == 0)
 			flist->files[0]->flags |= FLAG_DIR_CREATED;
 
-		if (verbose)
+		if (INFO_GTE(NAME, 1))
 			rprintf(FINFO, "created directory %s\n", dest_path);
 
 		if (dry_run) {
@@ -637,48 +706,63 @@ static char *get_local_name(struct file_list *flist, char *dest_path)
 static void check_alt_basis_dirs(void)
 {
 	STRUCT_STAT st;
-	char **dir_p, *slash = strrchr(curr_dir, '/');
+	char *slash = strrchr(curr_dir, '/');
+	int j;
 
-	for (dir_p = basis_dir; *dir_p; dir_p++) {
-		if (dry_run > 1 && **dir_p != '/') {
-			int len = curr_dir_len + 1 + strlen(*dir_p) + 1;
+	for (j = 0; j < basis_dir_cnt; j++) {
+		char *bdir = basis_dir[j];
+		int bd_len = strlen(bdir);
+		if (bd_len > 1 && bdir[bd_len-1] == '/')
+			bdir[--bd_len] = '\0';
+		if (dry_run > 1 && *bdir != '/') {
+			int len = curr_dir_len + 1 + bd_len + 1;
 			char *new = new_array(char, len);
 			if (!new)
 				out_of_memory("check_alt_basis_dirs");
-			if (slash && strncmp(*dir_p, "../", 3) == 0) {
+			if (slash && strncmp(bdir, "../", 3) == 0) {
 			    /* We want to remove only one leading "../" prefix for
 			     * the directory we couldn't create in dry-run mode:
 			     * this ensures that any other ".." references get
 			     * evaluated the same as they would for a live copy. */
 			    *slash = '\0';
-			    pathjoin(new, len, curr_dir, *dir_p + 3);
+			    pathjoin(new, len, curr_dir, bdir + 3);
 			    *slash = '/';
 			} else
-			    pathjoin(new, len, curr_dir, *dir_p);
-			*dir_p = new;
-		}
-		if (do_stat(*dir_p, &st) < 0) {
-			rprintf(FWARNING, "%s arg does not exist: %s\n",
-				dest_option, *dir_p);
-		} else if (!S_ISDIR(st.st_mode)) {
-			rprintf(FWARNING, "%s arg is not a dir: %s\n",
-				dest_option, *dir_p);
+			    pathjoin(new, len, curr_dir, bdir);
+			basis_dir[j] = bdir = new;
 		}
+		if (do_stat(bdir, &st) < 0)
+			rprintf(FWARNING, "%s arg does not exist: %s\n", dest_option, bdir);
+		else if (!S_ISDIR(st.st_mode))
+			rprintf(FWARNING, "%s arg is not a dir: %s\n", dest_option, bdir);
 	}
 }
 
 /* This is only called by the sender. */
-static void read_final_goodbye(int f_in)
+static void read_final_goodbye(int f_in, int f_out)
 {
 	int i, iflags, xlen;
 	uchar fnamecmp_type;
 	char xname[MAXPATHLEN];
 
+	shutting_down = True;
+
 	if (protocol_version < 29)
 		i = read_int(f_in);
 	else {
-		i = read_ndx_and_attrs(f_in, &iflags, &fnamecmp_type,
-				       xname, &xlen);
+		i = read_ndx_and_attrs(f_in, f_out, &iflags, &fnamecmp_type, xname, &xlen);
+		if (protocol_version >= 31 && i == NDX_DONE) {
+			if (am_sender)
+				write_ndx(f_out, NDX_DONE);
+			else {
+				if (batch_gen_fd >= 0) {
+					while (read_int(batch_gen_fd) != NDX_DEL_STATS) {}
+					read_del_stats(batch_gen_fd);
+				}
+				write_int(f_out, NDX_DONE);
+			}
+			i = read_ndx_and_attrs(f_in, f_out, &iflags, &fnamecmp_type, xname, &xlen);
+		}
 	}
 
 	if (i != NDX_DONE) {
@@ -693,17 +777,15 @@ static void do_server_sender(int f_in, int f_out, int argc, char *argv[])
 	struct file_list *flist;
 	char *dir = argv[0];
 
-	if (verbose > 2) {
-		rprintf(FINFO, "server_sender starting pid=%ld\n",
-			(long)getpid());
-	}
+	if (DEBUG_GTE(SEND, 1))
+		rprintf(FINFO, "server_sender starting pid=%d\n", (int)getpid());
 
 	if (am_daemon && lp_write_only(module_id)) {
 		rprintf(FERROR, "ERROR: module is write only\n");
 		exit_cleanup(RERR_SYNTAX);
 		return;
 	}
-	if (am_daemon && lp_read_only(module_id) && remove_source_files) {
+	if (am_daemon && read_only && remove_source_files) {
 		rprintf(FERROR,
 		    "ERROR: --remove-%s-files cannot be used with a read-only module\n",
 		    remove_source_files == 1 ? "source" : "sent");
@@ -728,8 +810,12 @@ static void do_server_sender(int f_in, int f_out, int argc, char *argv[])
 	}
 
 	flist = send_file_list(f_out,argc,argv);
-	if (!flist || flist->used == 0)
+	if (!flist || flist->used == 0) {
+		/* Make sure input buffering is off so we can't hang in noop_io_until_death(). */
+		io_end_buffering_in(0);
+		/* TODO:  we should really exit in a more controlled manner. */
 		exit_cleanup(0);
+	}
 
 	io_start_buffering_in(f_in);
 
@@ -737,7 +823,7 @@ static void do_server_sender(int f_in, int f_out, int argc, char *argv[])
 	io_flush(FULL_FLUSH);
 	handle_stats(f_out);
 	if (protocol_version >= 24)
-		read_final_goodbye(f_in);
+		read_final_goodbye(f_in, f_out);
 	io_flush(FULL_FLUSH);
 	exit_cleanup(0);
 }
@@ -763,7 +849,17 @@ static int do_recv(int f_in, int f_out, char *local_name)
 		exit_cleanup(RERR_IPC);
 	}
 
-	io_flush(NORMAL_FLUSH);
+	if (backup_dir) {
+		int ret = make_path(backup_dir_buf, MKP_DROP_NAME); /* drops trailing slash */
+		if (ret < 0)
+			exit_cleanup(RERR_SYNTAX);
+		if (ret)
+			rprintf(FINFO, "Created backup_dir %s\n", backup_dir_buf);
+		else if (INFO_GTE(BACKUP, 1))
+			rprintf(FINFO, "backup_dir is %s\n", backup_dir_buf);
+	}
+
+	io_flush(FULL_FLUSH);
 
 	if ((pid = do_fork()) == -1) {
 		rsyserr(FERROR, errno, "fork failed in do_recv");
@@ -772,38 +868,43 @@ static int do_recv(int f_in, int f_out, char *local_name)
 
 	if (pid == 0) {
 		am_receiver = 1;
+		send_msgs_to_gen = am_server;
 
 		close(error_pipe[0]);
+
+		/* We can't let two processes write to the socket at one time. */
+		io_end_multiplex_out(MPLX_SWITCHING);
 		if (f_in != f_out)
 			close(f_out);
+		sock_f_out = -1;
+		f_out = error_pipe[1];
 
-		/* we can't let two processes write to the socket at one time */
-		io_end_multiplex_out();
+		bwlimit_writemax = 0; /* receiver doesn't need to do this */
 
-		/* set place to send errors */
-		set_msg_fd_out(error_pipe[1]);
-		io_start_buffering_out(error_pipe[1]);
+		if (read_batch)
+			io_start_buffering_in(f_in);
+		io_start_multiplex_out(f_out);
 
-		recv_files(f_in, local_name);
+		recv_files(f_in, f_out, local_name);
 		io_flush(FULL_FLUSH);
 		handle_stats(f_in);
 
-		send_msg(MSG_DONE, "", 1, 0);
-		write_varlong(error_pipe[1], stats.total_read, 3);
+		if (output_needs_newline) {
+			fputc('\n', stdout);
+			output_needs_newline = 0;
+		}
+
+		write_int(f_out, NDX_DONE);
+		send_msg(MSG_STATS, (char*)&stats.total_read, sizeof stats.total_read, 0);
 		io_flush(FULL_FLUSH);
 
 		/* Handle any keep-alive packets from the post-processing work
 		 * that the generator does. */
 		if (protocol_version >= 29) {
-			int iflags, xlen;
-			uchar fnamecmp_type;
-			char xname[MAXPATHLEN];
-
 			kluge_around_eof = -1;
 
 			/* This should only get stopped via a USR2 signal. */
-			read_ndx_and_attrs(f_in, &iflags, &fnamecmp_type,
-					   xname, &xlen);
+			read_final_goodbye(f_in, f_out);
 
 			rprintf(FERROR, "Invalid packet at end of run [%s]\n",
 				who_am_i());
@@ -818,19 +919,20 @@ static int do_recv(int f_in, int f_out, char *local_name)
 	}
 
 	am_generator = 1;
+	flist_receiving_enabled = True;
 
-	io_end_multiplex_in();
+	io_end_multiplex_in(MPLX_SWITCHING);
 	if (write_batch && !am_server)
 		stop_write_batch();
 
 	close(error_pipe[1]);
 	if (f_in != f_out)
 		close(f_in);
+	sock_f_in = -1;
+	f_in = error_pipe[0];
 
 	io_start_buffering_out(f_out);
-
-	set_msg_fd_in(error_pipe[0]);
-	io_start_buffering_in(error_pipe[0]);
+	io_start_multiplex_in(f_in);
 
 #ifdef SUPPORT_HARD_LINKS
 	if (preserve_hard_links && inc_recurse) {
@@ -844,13 +946,13 @@ static int do_recv(int f_in, int f_out, char *local_name)
 
 	handle_stats(-1);
 	io_flush(FULL_FLUSH);
+	shutting_down = True;
 	if (protocol_version >= 24) {
 		/* send a final goodbye message */
 		write_ndx(f_out, NDX_DONE);
 	}
 	io_flush(FULL_FLUSH);
 
-	set_msg_fd_in(-1);
 	kill(pid, SIGUSR2);
 	wait_process_with_flush(pid, &exit_code);
 	return exit_code;
@@ -861,20 +963,20 @@ static void do_server_recv(int f_in, int f_out, int argc, char *argv[])
 	int exit_code;
 	struct file_list *flist;
 	char *local_name = NULL;
-	int save_verbose = verbose;
+	int negated_levels;
 
-	if (filesfrom_fd >= 0) {
+	if (filesfrom_fd >= 0 && !msgs2stderr && protocol_version < 31) {
 		/* We can't mix messages with files-from data on the socket,
-		 * so temporarily turn off verbose messages. */
-		verbose = 0;
-	}
+		 * so temporarily turn off info/debug messages. */
+		negate_output_levels();
+		negated_levels = 1;
+	} else
+		negated_levels = 0;
 
-	if (verbose > 2) {
-		rprintf(FINFO, "server_recv(%d) starting pid=%ld\n",
-			argc, (long)getpid());
-	}
+	if (DEBUG_GTE(RECV, 1))
+		rprintf(FINFO, "server_recv(%d) starting pid=%d\n", argc, (int)getpid());
 
-	if (am_daemon && lp_read_only(module_id)) {
+	if (am_daemon && read_only) {
 		rprintf(FERROR,"ERROR: module is read only\n");
 		exit_cleanup(RERR_SYNTAX);
 		return;
@@ -892,7 +994,7 @@ static void do_server_recv(int f_in, int f_out, int argc, char *argv[])
 	}
 
 	if (protocol_version >= 30)
-		io_start_multiplex_in();
+		io_start_multiplex_in(f_in);
 	else
 		io_start_buffering_in(f_in);
 	recv_filter_list(f_in);
@@ -903,7 +1005,7 @@ static void do_server_recv(int f_in, int f_out, int argc, char *argv[])
 		 * need the IO routines to automatically write out the names
 		 * onto our f_out socket as we read the file-list.  This
 		 * avoids both deadlock and extra delays/buffers. */
-		io_set_filesfrom_fds(filesfrom_fd, f_out);
+		start_filesfrom_forwarding(filesfrom_fd);
 		filesfrom_fd = -1;
 	}
 
@@ -914,7 +1016,9 @@ static void do_server_recv(int f_in, int f_out, int argc, char *argv[])
 	}
 	if (inc_recurse && file_total == 1)
 		recv_additional_file_list(f_in);
-	verbose = save_verbose;
+
+	if (negated_levels)
+		negate_output_levels();
 
 	if (argc > 0)
 		local_name = get_local_name(flist,argv[0]);
@@ -932,7 +1036,7 @@ static void do_server_recv(int f_in, int f_out, int argc, char *argv[])
 
 	if (daemon_filter_list.head) {
 		char **dir_p;
-		struct filter_list_struct *elp = &daemon_filter_list;
+		filter_rule_list *elp = &daemon_filter_list;
 
 		for (dir_p = basis_dir; *dir_p; dir_p++) {
 			char *dir = *dir_p;
@@ -971,12 +1075,16 @@ void start_server(int f_in, int f_out, int argc, char *argv[])
 	setup_protocol(f_out, f_in);
 
 	if (protocol_version >= 23)
-		io_start_multiplex_out();
+		io_start_multiplex_out(f_out);
+	if (am_daemon && io_timeout && protocol_version >= 31)
+		send_msg_int(MSG_IO_TIMEOUT, io_timeout);
 
 	if (am_sender) {
 		keep_dirlinks = 0; /* Must be disabled on the sender. */
 		if (need_messages_from_generator)
-			io_start_multiplex_in();
+			io_start_multiplex_in(f_in);
+		else
+			io_start_buffering_in(f_in);
 		recv_filter_list(f_in);
 		do_server_sender(f_in, f_out, argc, argv);
 	} else
@@ -984,11 +1092,8 @@ void start_server(int f_in, int f_out, int argc, char *argv[])
 	exit_cleanup(0);
 }
 
-
-/*
- * This is called once the connection has been negotiated.  It is used
- * for rsyncd, remote-shell, and local connections.
- */
+/* This is called once the connection has been negotiated.  It is used
+ * for rsyncd, remote-shell, and local connections. */
 int client_run(int f_in, int f_out, pid_t pid, int argc, char *argv[])
 {
 	struct file_list *flist = NULL;
@@ -1016,12 +1121,20 @@ int client_run(int f_in, int f_out, pid_t pid, int argc, char *argv[])
 
 	if (am_sender) {
 		keep_dirlinks = 0; /* Must be disabled on the sender. */
+
+		if (always_checksum
+		 && (log_format_has(stdout_format, 'C')
+		  || log_format_has(logfile_format, 'C')))
+			sender_keeps_checksum = 1;
+
 		if (protocol_version >= 30)
-			io_start_multiplex_out();
+			io_start_multiplex_out(f_out);
 		else
 			io_start_buffering_out(f_out);
-		if (!filesfrom_host)
-			set_msg_fd_in(f_in);
+		if (protocol_version >= 31 || (!filesfrom_host && protocol_version >= 23))
+			io_start_multiplex_in(f_in);
+		else
+			io_start_buffering_in(f_in);
 		send_filter_list(f_out);
 		if (filesfrom_host)
 			filesfrom_fd = f_in;
@@ -1029,20 +1142,20 @@ int client_run(int f_in, int f_out, pid_t pid, int argc, char *argv[])
 		if (write_batch && !am_server)
 			start_write_batch(f_out);
 		flist = send_file_list(f_out, argc, argv);
-		if (verbose > 3)
+		if (DEBUG_GTE(FLIST, 3))
 			rprintf(FINFO,"file list sent\n");
 
-		if (protocol_version >= 23)
-			io_start_multiplex_in();
+		if (protocol_version < 31 && filesfrom_host && protocol_version >= 23)
+			io_start_multiplex_in(f_in);
 
 		io_flush(NORMAL_FLUSH);
 		send_files(f_in, f_out);
 		io_flush(FULL_FLUSH);
 		handle_stats(-1);
 		if (protocol_version >= 24)
-			read_final_goodbye(f_in);
+			read_final_goodbye(f_in, f_out);
 		if (pid != -1) {
-			if (verbose > 3)
+			if (DEBUG_GTE(EXIT, 2))
 				rprintf(FINFO,"client_run waiting on %d\n", (int) pid);
 			io_flush(FULL_FLUSH);
 			wait_process_with_flush(pid, &exit_code);
@@ -1054,15 +1167,17 @@ int client_run(int f_in, int f_out, pid_t pid, int argc, char *argv[])
 
 	if (!read_batch) {
 		if (protocol_version >= 23)
-			io_start_multiplex_in();
+			io_start_multiplex_in(f_in);
 		if (need_messages_from_generator)
-			io_start_multiplex_out();
+			io_start_multiplex_out(f_out);
+		else
+			io_start_buffering_out(f_out);
 	}
 
 	send_filter_list(read_batch ? -1 : f_out);
 
 	if (filesfrom_fd >= 0) {
-		io_set_filesfrom_fds(filesfrom_fd, f_out);
+		start_filesfrom_forwarding(filesfrom_fd);
 		filesfrom_fd = -1;
 	}
 
@@ -1084,7 +1199,7 @@ int client_run(int f_in, int f_out, pid_t pid, int argc, char *argv[])
 	}
 
 	if (pid != -1) {
-		if (verbose > 3)
+		if (DEBUG_GTE(RECV, 1))
 			rprintf(FINFO,"client_run2 waiting on %d\n", (int) pid);
 		io_flush(FULL_FLUSH);
 		wait_process_with_flush(pid, &exit_code);
@@ -1109,14 +1224,12 @@ static int copy_argv(char *argv[])
 }
 
 
-/**
- * Start a client for either type of remote connection.  Work out
+/* Start a client for either type of remote connection.  Work out
  * whether the arguments request a remote shell or rsyncd connection,
  * and call the appropriate connection function, then run_client.
  *
  * Calls either start_socket_client (for sockets) or do_cmd and
- * client_run (for ssh).
- **/
+ * client_run (for ssh). */
 static int start_client(int argc, char *argv[])
 {
 	char *p, *shell_machine = NULL, *shell_user = NULL;
@@ -1203,6 +1316,9 @@ static int start_client(int argc, char *argv[])
 		remote_argc = argc = 1;
 	}
 
+	if (!rsync_port && remote_argc && !**remote_argv) /* Turn an empty arg into a dot dir. */
+		*remote_argv = ".";
+
 	if (am_sender) {
 		char *dummy_host;
 		int dummy_port = rsync_port;
@@ -1238,6 +1354,8 @@ static int start_client(int argc, char *argv[])
 					rprintf(FERROR, "All source args must use the same port number.\n");
 				exit_cleanup(RERR_SYNTAX);
 			}
+			if (!rsync_port && !*arg) /* Turn an empty arg into a dot dir. */
+				arg = ".";
 			remote_argv[i] = arg;
 		}
 	}
@@ -1266,10 +1384,10 @@ static int start_client(int argc, char *argv[])
 		}
 	}
 
-	if (verbose > 3) {
+	if (DEBUG_GTE(CMD, 2)) {
 		rprintf(FINFO,"cmd=%s machine=%s user=%s path=%s\n",
 			NS(shell_cmd), NS(shell_machine), NS(shell_user),
-			remote_argv ? NS(remote_argv[0]) : "");
+			NS(remote_argv[0]));
 	}
 
 	pid = do_cmd(shell_cmd, shell_machine, shell_user, remote_argv, remote_argc,
@@ -1372,10 +1490,9 @@ const char *get_panic_action(void)
 static RETSIGTYPE rsync_panic_handler(UNUSED(int whatsig))
 {
 	char cmd_buf[300];
-	int ret;
+	int ret, pid_int = getpid();
 
-	snprintf(cmd_buf, sizeof cmd_buf, get_panic_action(),
-		 getpid(), getpid());
+	snprintf(cmd_buf, sizeof cmd_buf, get_panic_action(), pid_int, pid_int);
 
 	/* Unless we failed to execute gdb, we allow the process to
 	 * continue.  I'm not sure if that's right. */
@@ -1411,6 +1528,7 @@ int main(int argc,char *argv[])
 
 	starttime = time(NULL);
 	our_uid = MY_UID();
+	our_gid = MY_GID();
 	am_root = our_uid == 0;
 
 	memset(&stats, 0, sizeof(stats));
@@ -1420,9 +1538,10 @@ int main(int argc,char *argv[])
 		exit_cleanup(RERR_SYNTAX);
 	}
 
-	/* we set a 0 umask so that correct file permissions can be
-	 * carried across */
-	orig_umask = umask(0);
+	/* Get the umask for use in permission calculations.  We no longer set
+	 * it to zero; that is ugly and pointless now that all the callers that
+	 * relied on it have been reeducated to work with default ACLs. */
+	umask(orig_umask = umask(0));
 
 #if defined CONFIG_LOCALE && defined HAVE_SETLOCALE
 	setlocale(LC_CTYPE, "");
